/*
 * Copyright 2010-2014 JetBrains s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package kotlin

public abstract class Number {
  public abstract fun toDouble() : Double
  public abstract fun toFloat() : Float
  public abstract fun toLong() : Long
  public abstract fun toInt() : Int
  public abstract fun toChar() : Char
  public abstract fun toShort() : Short
  public abstract fun toByte() : Byte
}

public class Double private () : Number, Comparable<Double>  {
  public override fun compareTo(other : Double)  : Int
  public fun compareTo(other : Float)  : Int
  public fun compareTo(other : Long)   : Int
  public fun compareTo(other : Int)    : Int
  public fun compareTo(other : Short)  : Int
  public fun compareTo(other : Byte)   : Int
  public fun compareTo(other : Char)   : Int

  public fun plus(other : Double) : Double
  public fun plus(other : Float)  : Double
  public fun plus(other : Long)   : Double
  public fun plus(other : Int)    : Double
  public fun plus(other : Short)  : Double
  public fun plus(other : Byte)   : Double
  public fun plus(other : Char)   : Double

  public fun minus(other : Double) : Double
  public fun minus(other : Float)  : Double
  public fun minus(other : Long)   : Double
  public fun minus(other : Int)    : Double
  public fun minus(other : Short)  : Double
  public fun minus(other : Byte)   : Double
  public fun minus(other : Char)   : Double

  public fun times(other : Double) : Double
  public fun times(other : Float)  : Double
  public fun times(other : Long)   : Double
  public fun times(other : Int)    : Double
  public fun times(other : Short)  : Double
  public fun times(other : Byte)   : Double
  public fun times(other : Char)   : Double

  public fun div(other : Double) : Double
  public fun div(other : Float)  : Double
  public fun div(other : Long)   : Double
  public fun div(other : Int)    : Double
  public fun div(other : Short)  : Double
  public fun div(other : Byte)   : Double
  public fun div(other : Char)   : Double

  public fun mod(other : Double) : Double
  public fun mod(other : Float)  : Double
  public fun mod(other : Long)   : Double
  public fun mod(other : Int)    : Double
  public fun mod(other : Short)  : Double
  public fun mod(other : Byte)   : Double

  public fun rangeTo(other : Double) : DoubleRange
  public fun rangeTo(other : Float)  : DoubleRange
  public fun rangeTo(other : Long)   : DoubleRange
  public fun rangeTo(other : Int)    : DoubleRange
  public fun rangeTo(other : Short)  : DoubleRange
  public fun rangeTo(other : Byte)   : DoubleRange
  public fun rangeTo(other : Char)   : DoubleRange

  public fun inc() : Double
  public fun dec() : Double
  public fun plus() : Double
  public fun minus() : Double

  public override fun toDouble() : Double
  public override fun toFloat() : Float
  public override fun toLong() : Long
  public override fun toInt() : Int
  public override fun toChar() : Char
  public override fun toShort() : Short
  public override fun toByte() : Byte
}

public class Float private () : Number, Comparable<Float>  {
  public fun compareTo(other : Double) : Int
  public override fun compareTo(other : Float) : Int
  public fun compareTo(other : Long)   : Int
  public fun compareTo(other : Int)    : Int
  public fun compareTo(other : Short)  : Int
  public fun compareTo(other : Byte)   : Int
  public fun compareTo(other : Char)   : Int

  public fun plus(other : Double) : Double
  public fun plus(other : Float)  : Float
  public fun plus(other : Long)   : Float
  public fun plus(other : Int)    : Float
  public fun plus(other : Short)  : Float
  public fun plus(other : Byte)   : Float
  public fun plus(other : Char)   : Float

  public fun minus(other : Double) : Double
  public fun minus(other : Float)  : Float
  public fun minus(other : Long)   : Float
  public fun minus(other : Int)    : Float
  public fun minus(other : Short)  : Float
  public fun minus(other : Byte)   : Float
  public fun minus(other : Char)   : Float

  public fun times(other : Double) : Double
  public fun times(other : Float)  : Float
  public fun times(other : Long)   : Float
  public fun times(other : Int)    : Float
  public fun times(other : Short)  : Float
  public fun times(other : Byte)   : Float
  public fun times(other : Char)   : Float

  public fun div(other : Double) : Double
  public fun div(other : Float)  : Float
  public fun div(other : Long)   : Float
  public fun div(other : Int)    : Float
  public fun div(other : Short)  : Float
  public fun div(other : Byte)   : Float
  public fun div(other : Char)   : Float

  public fun mod(other : Double) : Double
  public fun mod(other : Float)  : Float
  public fun mod(other : Long)   : Float
  public fun mod(other : Int)    : Float
  public fun mod(other : Short)  : Float
  public fun mod(other : Byte)   : Float
  public fun mod(other : Char)   : Float

  public fun rangeTo(other : Double) : DoubleRange
  public fun rangeTo(other : Float)  : FloatRange
  public fun rangeTo(other : Long)   : FloatRange
  public fun rangeTo(other : Int)    : FloatRange
  public fun rangeTo(other : Short)  : FloatRange
  public fun rangeTo(other : Byte)   : FloatRange
  public fun rangeTo(other : Char)   : FloatRange

  public fun inc() : Float
  public fun dec() : Float
  public fun plus() : Float
  public fun minus() : Float

  public override fun toDouble() : Double
  public override fun toFloat() : Float
  public override fun toLong() : Long
  public override fun toInt() : Int
  public override fun toChar() : Char
  public override fun toShort() : Short
  public override fun toByte() : Byte
}

public class Long private () : Number, Comparable<Long> {
  public fun compareTo(other : Double) : Int
  public fun compareTo(other : Float)  : Int
  public override fun compareTo(other : Long)   : Int
  public fun compareTo(other : Int)    : Int
  public fun compareTo(other : Short)  : Int
  public fun compareTo(other : Byte)   : Int
  public fun compareTo(other : Char)   : Int

  public fun plus(other : Double) : Double
  public fun plus(other : Float)  : Float
  public fun plus(other : Long)   : Long
  public fun plus(other : Int)    : Long
  public fun plus(other : Short)  : Long
  public fun plus(other : Byte)   : Long
  public fun plus(other : Char)   : Long

  public fun minus(other : Double) : Double
  public fun minus(other : Float)  : Float
  public fun minus(other : Long)   : Long
  public fun minus(other : Int)    : Long
  public fun minus(other : Short)  : Long
  public fun minus(other : Byte)   : Long
  public fun minus(other : Char)   : Long

  public fun times(other : Double) : Double
  public fun times(other : Float)  : Float
  public fun times(other : Long)   : Long
  public fun times(other : Int)    : Long
  public fun times(other : Short)  : Long
  public fun times(other : Byte)   : Long
  public fun times(other : Char)   : Long

  public fun div(other : Double) : Double
  public fun div(other : Float)  : Float
  public fun div(other : Long)   : Long
  public fun div(other : Int)    : Long
  public fun div(other : Short)  : Long
  public fun div(other : Byte)   : Long
  public fun div(other : Char)   : Long

  public fun mod(other : Double) : Double
  public fun mod(other : Float)  : Float
  public fun mod(other : Long)   : Long
  public fun mod(other : Int)    : Long
  public fun mod(other : Short)  : Long
  public fun mod(other : Byte)   : Long
  public fun mod(other : Char)   : Long

  public fun rangeTo(other : Double) : DoubleRange
  public fun rangeTo(other : Float)  : FloatRange
  public fun rangeTo(other : Long)   : LongRange
  public fun rangeTo(other : Int)    : LongRange
  public fun rangeTo(other : Short)  : LongRange
  public fun rangeTo(other : Byte)   : LongRange
  public fun rangeTo(other : Char)   : LongRange

  public fun inc() : Long
  public fun dec() : Long
  public fun plus() : Long
  public fun minus() : Long

  public fun shl(bits : Int) : Long
  public fun shr(bits : Int) : Long
  public fun ushr(bits : Int) : Long
  public fun and(other : Long) : Long
  public fun or(other : Long) : Long
  public fun xor(other : Long) : Long
  public fun inv() : Long

  public override fun toDouble() : Double
  public override fun toFloat() : Float
  public override fun toLong() : Long
  public override fun toInt() : Int
  public override fun toChar() : Char
  public override fun toShort() : Short
  public override fun toByte() : Byte
}

public class Int private () : Number, Comparable<Int> {
  public fun compareTo(other : Double) : Int
  public fun compareTo(other : Float)  : Int
  public fun compareTo(other : Long)   : Int
  public override fun compareTo(other : Int)    : Int
  public fun compareTo(other : Short)  : Int
  public fun compareTo(other : Byte)   : Int
  public fun compareTo(other : Char)   : Int

  public fun plus(other : Double) : Double
  public fun plus(other : Float)  : Float
  public fun plus(other : Long)   : Long
  public fun plus(other : Int)    : Int
  public fun plus(other : Short)  : Int
  public fun plus(other : Byte)   : Int
  public fun plus(other : Char)   : Int

  public fun minus(other : Double) : Double
  public fun minus(other : Float)  : Float
  public fun minus(other : Long)   : Long
  public fun minus(other : Int)    : Int
  public fun minus(other : Short)  : Int
  public fun minus(other : Byte)   : Int
  public fun minus(other : Char)   : Int

  public fun times(other : Double) : Double
  public fun times(other : Float)  : Float
  public fun times(other : Long)   : Long
  public fun times(other : Int)    : Int
  public fun times(other : Short)  : Int
  public fun times(other : Byte)   : Int
  public fun times(other : Char)   : Int

  public fun div(other : Double) : Double
  public fun div(other : Float)  : Float
  public fun div(other : Long)   : Long
  public fun div(other : Int)    : Int
  public fun div(other : Short)  : Int
  public fun div(other : Byte)   : Int
  public fun div(other : Char)   : Int

  public fun mod(other : Double) : Double
  public fun mod(other : Float)  : Float
  public fun mod(other : Long)   : Long
  public fun mod(other : Int)    : Int
  public fun mod(other : Short)  : Int
  public fun mod(other : Byte)   : Int
  public fun mod(other : Char)   : Int

  public fun rangeTo(other : Double) : DoubleRange
  public fun rangeTo(other : Float)  : FloatRange
  public fun rangeTo(other : Long)   : LongRange
  public fun rangeTo(other : Int)    : IntRange
  public fun rangeTo(other : Short)  : IntRange
  public fun rangeTo(other : Byte)   : IntRange
  public fun rangeTo(other : Char)   : IntRange

  public fun inc() : Int
  public fun dec() : Int
  public fun plus() : Int
  public fun minus() : Int

  public fun shl(bits : Int) : Int
  public fun shr(bits : Int) : Int
  public fun ushr(bits : Int) : Int
  public fun and(other : Int) : Int
  public fun or(other : Int) : Int
  public fun xor(other : Int) : Int
  public fun inv() : Int

  public override fun toDouble() : Double
  public override fun toFloat() : Float
  public override fun toLong() : Long
  public override fun toInt() : Int
  public override fun toChar() : Char
  public override fun toShort() : Short
  public override fun toByte() : Byte
}

public class Short private () : Number, Comparable<Short> {
  public fun compareTo(other : Double) : Int
  public fun compareTo(other : Float)  : Int
  public fun compareTo(other : Long)    : Int
  public fun compareTo(other : Int)    : Int
  public override fun compareTo(other : Short)    : Int
  public fun compareTo(other : Byte)   : Int
  public fun compareTo(other : Char)   : Int

  public fun plus(other : Double) : Double
  public fun plus(other : Float)  : Float
  public fun plus(other : Long)   : Long
  public fun plus(other : Int)    : Int
  public fun plus(other : Short)  : Int
  public fun plus(other : Byte)   : Int
  public fun plus(other : Char)   : Int

  public fun minus(other : Double) : Double
  public fun minus(other : Float)  : Float
  public fun minus(other : Long)   : Long
  public fun minus(other : Int)    : Int
  public fun minus(other : Short)  : Int
  public fun minus(other : Byte)   : Int
  public fun minus(other : Char)   : Int

  public fun times(other : Double) : Double
  public fun times(other : Float)  : Float
  public fun times(other : Long)   : Long
  public fun times(other : Int)    : Int
  public fun times(other : Short)  : Int
  public fun times(other : Byte)   : Int
  public fun times(other : Char)   : Int

  public fun div(other : Double) : Double
  public fun div(other : Float)  : Float
  public fun div(other : Long)   : Long
  public fun div(other : Int)    : Int
  public fun div(other : Short)  : Int
  public fun div(other : Byte)   : Int
  public fun div(other : Char)   : Int

  public fun mod(other : Double) : Double
  public fun mod(other : Float)  : Float
  public fun mod(other : Long)   : Long
  public fun mod(other : Int)    : Int
  public fun mod(other : Short)  : Int
  public fun mod(other : Byte)   : Int
  public fun mod(other : Char)   : Int

  public fun rangeTo(other : Double) : DoubleRange
  public fun rangeTo(other : Float)  : FloatRange
  public fun rangeTo(other : Long)   : LongRange
  public fun rangeTo(other : Int)    : IntRange
  public fun rangeTo(other : Short)  : ShortRange
  public fun rangeTo(other : Byte)   : ShortRange
  public fun rangeTo(other : Char)   : ShortRange

  public fun inc() : Short
  public fun dec() : Short
  public fun plus() : Int
  public fun minus() : Int

  public override fun toDouble() : Double
  public override fun toFloat() : Float
  public override fun toLong() : Long
  public override fun toInt() : Int
  public override fun toChar() : Char
  public override fun toShort() : Short
  public override fun toByte() : Byte
}

public class Byte private () : Number, Comparable<Byte> {
  public fun compareTo(other : Double) : Int
  public fun compareTo(other : Float)  : Int
  public fun compareTo(other : Long)    : Int
  public fun compareTo(other : Int)    : Int
  public fun compareTo(other : Short)  : Int
  public fun compareTo(other : Char)   : Int
  public override fun compareTo(other : Byte)   : Int

  public fun plus(other : Double) : Double
  public fun plus(other : Float)  : Float
  public fun plus(other : Long)   : Long
  public fun plus(other : Int)    : Int
  public fun plus(other : Short)  : Int
  public fun plus(other : Byte)   : Int
  public fun plus(other : Char)   : Int

  public fun minus(other : Double) : Double
  public fun minus(other : Float)  : Float
  public fun minus(other : Long)   : Long
  public fun minus(other : Int)    : Int
  public fun minus(other : Short)  : Int
  public fun minus(other : Byte)   : Int
  public fun minus(other : Char)   : Int

  public fun times(other : Double) : Double
  public fun times(other : Float)  : Float
  public fun times(other : Long)   : Long
  public fun times(other : Int)    : Int
  public fun times(other : Short)  : Int
  public fun times(other : Byte)   : Int
  public fun times(other : Char)   : Int

  public fun div(other : Double) : Double
  public fun div(other : Float)  : Float
  public fun div(other : Long)   : Long
  public fun div(other : Int)    : Int
  public fun div(other : Short)  : Int
  public fun div(other : Byte)   : Int
  public fun div(other : Char)   : Int

  public fun mod(other : Double) : Double
  public fun mod(other : Float)  : Float
  public fun mod(other : Long)   : Long
  public fun mod(other : Int)    : Int
  public fun mod(other : Short)  : Int
  public fun mod(other : Byte)   : Int
  public fun mod(other : Char)   : Int

  public fun rangeTo(other : Double) : DoubleRange
  public fun rangeTo(other : Float)  : FloatRange
  public fun rangeTo(other : Long)   : LongRange
  public fun rangeTo(other : Int)    : IntRange
  public fun rangeTo(other : Short)  : ShortRange
  public fun rangeTo(other : Byte)   : ByteRange
  public fun rangeTo(other : Char)   : CharRange

  public fun inc() : Byte
  public fun dec() : Byte
  public fun plus() : Int
  public fun minus() : Int

  public override fun toDouble() : Double
  public override fun toFloat() : Float
  public override fun toLong() : Long
  public override fun toInt() : Int
  public override fun toChar() : Char
  public override fun toShort() : Short
  public override fun toByte() : Byte
}
